@@ -832,7 +832,7 @@ aiounicast_select::~aiounicast_select
 	for (size_t i = 0; i < n; i++)
 	{
 		delete [] buf_in[i];
-		for (size_t mm = 0; mm < buf_mpz[i].size(); mm++)
+		while (buf_mpz[i].size())
 		{
 			mpz_clear(buf_mpz[i].front());
 			delete [] buf_mpz[i].front();
