@@ -1,7 +1,8 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2005, 2006, 2007  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2005, 2006, 2007, 2009,
+               2015, 2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +19,14 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *******************************************************************************/
 
+// include headers
+#ifdef HAVE_CONFIG_H
+	#include "libTMCG_config.h"
+#endif
+#include <libTMCG.hh>
+
+#ifdef FORKING
+
 #include <sstream>
 #include <vector>
 #include <algorithm>
@@ -27,7 +36,6 @@
 #include <sys/wait.h>
 
 #include "test_helper.h"
-#include <libTMCG.hh>
 #include "pipestream.hh"
 
 #undef NDEBUG
@@ -87,6 +95,11 @@ void check
 			vtmf2->KeyGenerationProtocol_PublishKey(*pipe_out);
 			assert(vtmf2->KeyGenerationProtocol_UpdateKey(*pipe_in));
 			vtmf2->KeyGenerationProtocol_Finalize();
+			*pipe_out << vtmf2->h_i << std::endl;
+			assert(vtmf2->KeyGenerationProtocol_ProveKey_interactive(*pipe_in, *pipe_out));
+			JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,	vtmf2->p, vtmf2->q, vtmf2->g, vtmf2->h);
+			assert(vtmf2->KeyGenerationProtocol_ProveKey_interactive_publiccoin(edcf, *pipe_in, *pipe_out));
+			delete edcf;
 			
 			SchindelhauerTMCG *tmcg = 
 				new SchindelhauerTMCG(16, 2, TMCG_MAX_TYPEBITS);
@@ -147,6 +160,17 @@ void check
 			assert(vtmf->KeyGenerationProtocol_UpdateKey(*pipe_in));
 			std::cout << "*.KeyGenerationProtocol_Finalize()" << std::endl;
 			vtmf->KeyGenerationProtocol_Finalize();
+			std::cout << "*.KeyGenerationProtocol_VerifyKey_interactive()" << std::endl;
+			mpz_t h_j;
+			mpz_init(h_j);
+			*pipe_in >> h_j;
+			std::cout << "h_j = " << h_j << std::endl;
+			assert(vtmf->KeyGenerationProtocol_VerifyKey_interactive(h_j, *pipe_in, *pipe_out));
+			JareckiLysyanskayaEDCF *edcf = new JareckiLysyanskayaEDCF(2, 0,	vtmf->p, vtmf->q, vtmf->g, vtmf->h);
+			std::cout << "*.KeyGenerationProtocol_VerifyKey_interactive_publiccoin()" << std::endl;
+			assert(vtmf->KeyGenerationProtocol_VerifyKey_interactive_publiccoin(h_j, edcf, *pipe_in, *pipe_out));
+			delete edcf;
+			mpz_clear(h_j);
 			
 			// TMCG/VTMF
 			std::cout << "TMCG/VTMF Encryption and decryption of cards" << std::endl;
@@ -155,6 +179,30 @@ void check
 			
 			TMCG_OpenStack<VTMF_Card> os;
 			TMCG_Stack<VTMF_Card> sA, sAB, sB;
+			std::cout << "A: test basic stack operations" << std::endl;
+			for (size_t i = 0; i < TMCG_MAX_CARDS; i++)
+			{
+				VTMF_Card c;
+				os.push(i, c);
+			}
+			assert(os.size() == TMCG_MAX_CARDS);
+			size_t j = (TMCG_MAX_CARDS - 1);
+			for (size_t i = 0; i < TMCG_MAX_CARDS; i++, j--)
+			{
+				VTMF_Card c;
+				size_t idx = TMCG_MAX_CARDS;
+				bool notempty = os.pop(idx, c);
+				assert(notempty);
+				assert(idx == j);
+			}
+			assert(os.empty());
+			for (size_t i = 0; i < TMCG_MAX_CARDS; i++)
+			{
+				VTMF_Card c;
+				os.push(i, c);
+			}
+			os.clear();
+			assert(os.empty());
 			TMCG_StackSecret<VTMF_CardSecret> ssA;
 			std::cout << "A: CreateOpenCard()" << std::endl;
 			for (size_t i = 0; i < TMCG_MAX_CARDS; i++)
@@ -222,26 +270,37 @@ int main
 {
 	std::stringstream oak, lej, lej2, foo, foo2, bar;
 	std::string v;
-	mpz_t fooo;
+	mpz_t fooo, barr;
 	
 	assert(init_libTMCG());
 	
 	BarnettSmartVTMF_dlog *vtmf, *vtmf2;
 	BarnettSmartVTMF_dlog_GroupQR *vtmf_qr, *vtmf2_qr;
-	
-	// create and check the common instance
-	oak << "1a1e4vngailcvh7j2tur8zts6fbkrcac6d3g0jdfvdggl9r7cr2v6zllf6b3z6xb6w0\
-nix8jsjlecgt8lwx1huihghkvqnbbgviiz0vg6ntckohpeac84n0co9czkw174ic47ifwd2189x26\
-609ce63xm5vddxvsbbevn4bxaiv2784d9335o38na680ay8apygtmnz" << std::endl;
-	oak << "n0p2ftq59aofqlrjexdmhww37nsdo5636jq09opxoq8amvlodjflhsspl5jzlgnlg0b\
-rgm9w9sp68emaygiqx98q8sfvbnnqfr9hifq3bwoac8up5642bi6c4ohsg0lk9623r7y6j0m4yj33\
-04o731yt2xooyxw5npftk5yn9fj3m26mjjku1mbn3405h45cz8etbz" << std::endl;
-	oak << "2" << std::endl << "2" << std::endl;
-	std::cout << "BarnettSmartVTMF_dlog_GroupQR(<Oakley Group 2>)" << std::endl;
+
+	// create and check the common instance <2048-bit MODP Group [RFC3526]>
+	mpz_init(barr);
+	mpz_set_str(barr, "FFFFFFFFFFFFFFFFC90FDAA22168C234C4C6628B80DC1CD1\
+29024E088A67CC74020BBEA63B139B22514A08798E3404DD\
+EF9519B3CD3A431B302B0A6DF25F14374FE1356D6D51C245\
+E485B576625E7EC6F44C42E9A637ED6B0BFF5CB6F406B7ED\
+EE386BFB5A899FA5AE9F24117C4B1FE649286651ECE45B3D\
+C2007CB8A163BF0598DA48361C55D39A69163FA8FD24CF5F\
+83655D23DCA3AD961C62F356208552BB9ED529077096966D\
+670C354E4ABC9804F1746C08CA18217C32905E462E36CE3B\
+E39E772C180E86039B2783A2EC07A28FB5C55DF06F4C52C9\
+DE2BCBF6955817183995497CEA956AE515D2261898FA0510\
+15728E5A8AACAA68FFFFFFFFFFFFFFFF", 16);
+	oak << barr << std::endl; // p
+	mpz_sub_ui(barr, barr, 1L);
+	mpz_fdiv_q_2exp(barr, barr, 1L);
+	oak << barr << std::endl; // q
+	oak << "2" << std::endl << "2" << std::endl; // g and k
+	std::cout << "BarnettSmartVTMF_dlog_GroupQR(<2048-bit MODP Group [RFC3526]>)" << std::endl;
 	vtmf_qr = new BarnettSmartVTMF_dlog_GroupQR(oak);
 	std::cout << "vtmf_qr.CheckGroup()" << std::endl;
 	assert(vtmf_qr->CheckGroup());
 	delete vtmf_qr;
+	mpz_clear(barr);
 	
 	// create and check the instance
 	std::cout << "BarnettSmartVTMF_dlog_GroupQR()" << std::endl;
@@ -313,8 +372,59 @@ rgm9w9sp68emaygiqx98q8sfvbnnqfr9hifq3bwoac8up5642bi6c4ohsg0lk9623r7y6j0m4yj33\
 	stop_clock();
 	std::cout << elapsed_time() << std::endl;
 	
+	// release the instances
+	delete vtmf, delete vtmf2;
+
+	// create and check the instance
+	std::cout << "BarnettSmartVTMF_dlog(canonical_g == true)" << std::endl;
+	vtmf = new BarnettSmartVTMF_dlog(TMCG_DDH_SIZE, TMCG_DLSE_SIZE, true);
+	std::cout << "vtmf.CheckGroup()" << std::endl;
+	assert(vtmf->CheckGroup());
+	
+	// large generator check
+	mpz_init(fooo), mpz_init(barr);
+	mpz_set(fooo, vtmf->g);
+	mpz_wrandomm(barr, vtmf->p);
+	mpz_powm(vtmf->g, barr, vtmf->k, vtmf->p);
+	assert(!vtmf->CheckGroup());
+	mpz_set(vtmf->g, fooo);
+	mpz_clear(fooo), mpz_clear(barr);
+	
+	// publish the instance
+	std::cout << "vtmf.PublishGroup(lej)" << std::endl;
+	vtmf->PublishGroup(lej);
+	
+	// create a clone of the instance
+	std::cout << "BarnettSmartVTMF_dlog(lej)" << std::endl;
+	vtmf2 = new BarnettSmartVTMF_dlog(lej);
+	
+	// publish the cloned instance
+	std::cout << "vtmf2.PublishGroup(lej2)" << std::endl;
+	vtmf2->PublishGroup(lej2);
+	std::cout << lej.str() << std::endl;
+	std::cout << "versus" << std::endl;
+	std::cout << lej2.str() << std::endl;
+	assert(lej.str() == lej2.str());
+	
+	// check the instances
+	start_clock();
+	check(vtmf, vtmf2);
+	stop_clock();
+	std::cout << elapsed_time() << std::endl;
+	
 	// release the instances
 	delete vtmf, delete vtmf2;
 	
 	return 0;
 }
+
+#else
+
+int main
+	(int argc, char **argv)
+{
+	std::cout << "test skipped" << std::endl;
+	return 77;
+}
+
+#endif
