@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of libTMCG.
 
- Copyright (C) 2005  Heiko Stamer <stamer@gaos.org>
+ Copyright (C) 2005, 2017  Heiko Stamer <HeikoStamer@gmx.net>
 
    libTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include "test_helper.h"
 
 clock_t start, stop;
+clock_t start_saved, stop_saved;
 char buf[50];
 
 void start_clock
@@ -35,6 +36,13 @@ void stop_clock
 	stop = clock();
 }
 
+void save_clock
+	()
+{
+	start_saved = start;
+	stop_saved = stop;
+}
+
 char *elapsed_time
 	()
 {
@@ -50,3 +58,40 @@ char *current_time
 		(((double) (clock() - start)) / CLOCKS_PER_SEC) * 1000);
 	return buf;
 }
+
+int compare_time
+	(clock_t diff)
+{
+	if (start < (stop - diff))
+		return 1;
+	else if (start == (stop - diff))
+		return 0;
+	else
+		return -1;
+}
+
+int compare_elapsed_time
+	(clock_t diff)
+{
+	clock_t elapsed = stop - start;
+	if (elapsed > diff)
+		return 1;
+	else if (elapsed == diff)
+		return 0;
+	else
+		return -1;
+}
+
+int compare_elapsed_time_saved
+	(clock_t diff)
+{
+	clock_t elapsed = stop - start;
+	clock_t elapsed_saved = stop_saved - start_saved;
+	if (elapsed < (elapsed_saved - diff))
+		return 1;
+	else if (elapsed == (elapsed_saved - diff))
+		return 0;
+	else
+		return -1;
+}
+
